package eu.nets.sis.idrights.demoapp.controller;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import no.bbs.trust.b2b.xmlapi.message.generated.SigAndProcRelations;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import eu.nets.sis.idrights.clientapi.exception.IDRightsClientException;
import eu.nets.sis.idrights.demoapp.service.IDRightsService;
import eu.nets.sis.idrights.demoapp.util.IDRightsUtil;
import no.bbs.trust.b2b.xmlapi.message.domain.B2BMessage;
import no.bbs.trust.b2b.xmlapi.message.generated.CompanyRole;
import no.bbs.trust.b2b.xmlapi.message.generated.InterpretedSigAndProcInfoTypeDef;


/**
 * 
 * Controller that handles various statuses and home page request of the application
 * by mapping the requests to the appropriate views
 *
 */
@Controller
public class ApplicationController {
	
	private static Logger logger = LogManager.getLogger(ApplicationController.class);
	private static final String GET_SIGNATURE_DATA = "getsignaturedata";
	private static final String VERIFY_SIGNATURE_DATA = "verifysignaturedata";
	
	@Autowired
	private IDRightsService idrightsService;
	
	/**
	 * Displays entry page (contains iframe)
	 * @return String value that represents the target HTML page
	 */
	@RequestMapping(value = "/idrights", method = RequestMethod.GET)
	public String home() {
		return "index";
	}

	/**
	 * Displays landing page with content (displayed inside an iframe in index page)
	 * 
	 * @return String value that represents the target HTML page
	 */
	@RequestMapping(value = "/iframepage", method = RequestMethod.GET)
	public String iframe() {
		return "iframepage";
	}
	
	/**
	 * Responsible to show result page based on user selection in UI 
	 * @param messageType value of the user selected button
	 * @return String value that represents the target HTML page
	 */
	@RequestMapping(value = "/service", method = RequestMethod.GET)
	public String service(@RequestParam("message_type") String messageType) {
		
		if (Arrays.asList(GET_SIGNATURE_DATA, VERIFY_SIGNATURE_DATA).contains(messageType)) {
			return messageType;
		} 
		logger.info("Unsupported messageType, hence moving to error page");
		return "error";
	}
	
	/**
	 * Displays the result page of Get Signature Data
	 * @param request HTTP request object
	 * @param model Form data in key value pair
	 * @return String value that represents the target HTML page
	 */
	@RequestMapping(value = "/getsignaturedatastatus", method = RequestMethod.GET)
	public String getSignatureDataStatus(HttpServletRequest request, ModelMap model) {
		
		String countryCode = request.getParameter("countrycode");
		String orgNo = request.getParameter("organizationNo");
		String headerText = request.getParameter("showheadertext");
		logger.info("headerText=" + headerText);
		
		B2BMessage response;
		try {
			response = idrightsService.getSPInfoByOrgXmlRequest(countryCode, orgNo, headerText);
		} catch (IDRightsClientException e) {
			model.put("errorcode", e.getErrorCode());
			model.put("errormessage", e.getMessage());
			return "error";
		}
		if (null != response) {
			List<CompanyRole> companyRoles = response.getTrustB2BMessage().getGetSPInfoByOrgResponse().getCompanyRoles().getCompanyRole();
			List<InterpretedSigAndProcInfoTypeDef> signatureList = response.getTrustB2BMessage().getGetSPInfoByOrgResponse().getInterpretedSPInfoList().getInterpretedSPInfo();
			List<InterpretedSigAndProcInfoTypeDef> signatures = new LinkedList<>();
			for (InterpretedSigAndProcInfoTypeDef sig : signatureList) {
				if ("Signature".equals(sig.getSignatureType().value())) { // Get only Signature data from list
					if (sig.getSigAndProcRelations() == null) {
						sig.setSigAndProcRelations(new SigAndProcRelations());
					}
					signatures.add(sig);
				}
			}
			String xmlResponse = IDRightsUtil.convertB2BMsgToString(response);
			if (null != xmlResponse) {
				model.put("organizationNo", orgNo);
				model.put("roles", companyRoles);
				model.put("signaturelist", signatures);
				model.put("idrightsresponse", IDRightsUtil.convertToXMLStringFormat(xmlResponse));
				return "getsignaturedatastatus";
			}
		}
		return "error";
	}
	
	/**
	 * Displays the result page of Verify Signature Data
	 * @param request HTTP request object
	 * @param model Form data in key value pair
	 * @return String value that represents the target HTML page
	 */
	@RequestMapping(value = "/verifysignaturedatastatus", method = RequestMethod.GET)
	public String verifySignatureDataStatus(HttpServletRequest request, ModelMap model) {
		
		String countryCode = request.getParameter("countrycode");
		String orgNo = request.getParameter("organizationNo");
		String ssnValues = request.getParameter("ssn");
		
		B2BMessage response;
		try {
			response = idrightsService.verifySPInfoXmlRequest(countryCode, orgNo, ssnValues);
		} catch (IDRightsClientException e) {
			model.put("errorcode", e.getErrorCode());
			model.put("errormessage", e.getMessage());
			return "error";
		}
		if (null != response) {
			String signatureType = response.getTrustB2BMessage().getVerifySignatureAndProcurationResponse().getSignatureType().value();
			String signatureConclusion = response.getTrustB2BMessage().getVerifySignatureAndProcurationResponse().getSignatureConclusion().value();
			String xmlResponse = IDRightsUtil.convertB2BMsgToString(response);
			if (null != xmlResponse) {
				model.put("organizationNo", orgNo);
				model.put("ssn", ssnValues);
				model.put("signaturetype", signatureType);
				model.put("signatureconclusion", signatureConclusion);
				model.put("idrightsresponse", IDRightsUtil.convertToXMLStringFormat(xmlResponse));
				return "verifysignaturedatastatus";
			}
		}
		return "error";
	}
	
	/**
	 * Responsible for showing error page on error
	 * 
	 * @return String value that represents the target HTML page
	 */
	@RequestMapping(value = "/error", method = RequestMethod.GET)
	@ExceptionHandler(Exception.class)
	public String defaultErrorHandler() {
		return "error";
	}

}
